#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <Performer/pf/pfChannel.h>
#include <Performer/pf/pfLightSource.h>
#include <Performer/pf/pfDCS.h>
#include <Performer/pf/pfTraverser.h>
#include <Performer/pf/pfLOD.h>
#include <Performer/pfdu.h>
#include <pfcave.h>

static void createScene(pfChannel *chan,int argc,char **argv);
static void loadObjects(int argc,char **argv,pfGroup *parent);
static int navigate(pfTraverser *trav,void *data);


int main(int argc,char **argv)
{
 pfInit();
 pfCAVEConfig(&argc,argv,NULL);
 pfConfig();
 pfCAVEInitChannels();
 if (CAVEConfig->Simulator)
	pfCAVEMasterChan()->getFStats()->setClass(
			PFSTATS_ALL^PFSTATSHW_ENGFXPIPE_FILL, PFSTATS_ON);
 else
	pfCAVEMasterChan()->getFStats()->setClass(PFSTATS_ALL, PFSTATS_OFF);
 createScene(pfCAVEMasterChan(),argc,argv);
 while (!CAVEgetbutton(CAVE_ESCKEY))
	{
	pfSync();
	pfCAVEPreFrame();
	pfFrame();
	pfCAVEPostFrame();
	}
 CAVEHalt();
 pfExit();
 return 0;
}


static void createScene(pfChannel *chan,int argc,char **argv)
{
 pfScene *scene;
 pfGeoState *gstate;
 pfDCS *dcs;
 scene = new pfScene;
 gstate = new pfGeoState;				/* Set up a base geostate for the scene */
 gstate->setMode(PFSTATE_ENLIGHTING, PF_ON);
 gstate->setMode(PFSTATE_CULLFACE, PFCF_OFF);
 scene->setGState(gstate);
 scene->addChild(new pfLightSource);			/* Add a default light */
 dcs = new pfDCS;					/* Create a DCS for navigation */
 scene->addChild(dcs);
 dcs->setTravFuncs(PFTRAV_APP,navigate,NULL);
 loadObjects(argc,argv,dcs);
 chan->setScene(scene);					/* Assign the scene to the channels */
}


static void loadObjects(int argc,char **argv,pfGroup *parent)
{
 int i;
 pfNode *obj;
 pfVec3 origin(0,0,0);
 pfLOD *lod = new pfLOD;
 lod->setCenter(origin);
 lod->setRange(0,0.1f);
 for (i=1; i<argc; i++)
	{
	obj = pfdLoadFile(argv[i]);			/* Load the object */
	if (obj)					/* If it succeeded, add the object to the graph */
		{
		lod->addChild(obj);
		lod->setRange(i,i*40.0f);
		lod->setTransition(i,10.0f);
		}
	else
		pfExit();
	}
 lod->setRange(i-1,100000.0f);
 parent->addChild(lod);
}


static int navigate(pfTraverser *trav,void *)
{
#define SPEED 1.0f
 pfDCS *navDcs = (pfDCS *) trav->getNode();		/* Get a pointer to the DCS node itself */
 float jx=CAVE_JOYSTICK_X,jy=CAVE_JOYSTICK_Y;
 if (fabs(jx) > 0.07)					/* Rotate if the joystick is pushed left or right */
	CAVENavRot(-1.0f*jx,'z');
 if (fabs(jy) > 0.07)					/* Translate if the joystick is pushed forward or back */
	{
	float w[3];
	CAVEGetVector(CAVE_WAND_FRONT,w);
	CAVENavTranslate(w[0]*jy*SPEED,w[1]*jy*SPEED,w[2]*jy*SPEED);
	}
 if ((CAVEBUTTON1) && (CAVEBUTTON3))			/* Reset navigation on secret handshake */
	CAVENavLoadIdentity();
 pfCAVEDCSNavTransform(navDcs);				/* Load the DCS with the latest navigation matrix */
 return PFTRAV_CONT;					/* Tell Performer to continue traversing */
}
