#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <Performer/pf/pfChannel.h>
#include <Performer/pf/pfLightSource.h>
#include <Performer/pf/pfDCS.h>
#include <Performer/pf/pfTraverser.h>
#include <Performer/pfdu.h>
#include <pfcave.h>

static void createScene(pfChannel *chan,int argc,char **argv);
static void loadObjects(int argc,char **argv,pfGroup *parent);
static void navigate(void);
static void applyNavigation(pfChannel *chan);


int main(int argc,char **argv)
{
 pfInit();
 pfCAVEConfig(&argc,argv,NULL);
 pfConfig();
 pfCAVEInitChannels();
 if (CAVEConfig->Simulator)
	pfCAVEMasterChan()->getFStats()->setClass(
			PFSTATS_ALL^PFSTATSHW_ENGFXPIPE_FILL, PFSTATS_ON);
 else
	pfCAVEMasterChan()->getFStats()->setClass(PFSTATS_ALL, PFSTATS_OFF);
 createScene(pfCAVEMasterChan(),argc,argv);
 while (!CAVEgetbutton(CAVE_ESCKEY))
	{
	navigate();				/* Do most of the work here, during the frame */
	pfSync();
	pfCAVEPreFrame();
	applyNavigation(pfCAVEMasterChan());	/* Apply the nav transform to the new ViewMat */
	pfFrame();
	pfCAVEPostFrame();
	}
 CAVEHalt();
 pfExit();
 return 0;
}


static void createScene(pfChannel *chan,int argc,char **argv)
{
 pfScene *scene;
 pfGeoState *gstate;
 scene = new pfScene;
 gstate = new pfGeoState;
 gstate->setMode(PFSTATE_ENLIGHTING, PF_ON);
 gstate->setMode(PFSTATE_CULLFACE, PFCF_OFF);
 scene->setGState(gstate);
 scene->addChild(new pfLightSource);
 loadObjects(argc,argv,scene);
 chan->setScene(scene);
}


static void loadObjects(int argc,char **argv,pfGroup *parent)
{
 int i;
 pfNode *obj;
 for (i=1; i<argc; i++)
	{
	obj = pfdLoadFile(argv[i]);
	if (obj)
		parent->addChild(obj);
	else
		pfExit();
	}
}


static void navigate(void)
{
#define SPEED 2.0f
 float jx=CAVE_JOYSTICK_X,jy=CAVE_JOYSTICK_Y;
 if (fabs(jx) > 0.125)
	CAVENavRot(-1.0f*jx,'z');
 if (fabs(jy) > 0.125)
	{
	float w[3];
	CAVEGetVector(CAVE_WAND_FRONT,w);
	CAVENavTranslate(w[0]*jy*SPEED,w[1]*jy*SPEED,w[2]*jy*SPEED);
	}
 if ((CAVEBUTTON1) && (CAVEBUTTON3))
	CAVENavLoadIdentity();
}

static void applyNavigation(pfChannel *chan)
{
 pfMatrix navmat,viewmat;
 CAVENavGetMatrix(navmat.mat);
 navmat.invertAff(navmat);
 chan->getViewMat(viewmat);
 viewmat *= navmat;
 chan->setViewMat(viewmat);
}
